searchNodes=[{"doc":"ExQ ExQ provides a way of queuing the execution of operations and aggregates all returned values similar to Ecto.Multi . Operations are queued and executed in FIFO order. iex&gt; pipeline = Q . new ( ) |&gt; Q . put ( :init , %{ test : &quot;setup&quot; } ) |&gt; Q . run ( :read , fn _ -&gt; { :ok , &quot;Once upon a time ...&quot; } end ) iex&gt; pipeline |&gt; Q . exec ( ) { :ok , %{ init : %{ test : &quot;setup&quot; } , read : &quot;Once upon a time ...&quot; } } iex&gt; pipeline |&gt; Q . run ( :write , fn %{ read : _read } -&gt; { :error , :write_failed } end ) |&gt; Q . exec ( ) { :error , :write , :write_failed , %{ init : %{ test : &quot;setup&quot; } , read : &quot;Once upon a time ...&quot; } }","ref":"readme.html","title":"README","type":"extras"},{"doc":"A major benefit of using ExQ over with is that the results of all steps before an error are available and the step which produced the error is also easily identifiable. user_id = 1234 with { :ok , user } &lt;- Blog . fetch_user ( user_id ) , { :ok , post } &lt;- Blog . create_post ( user , &quot;test&quot; ) do broadcast ( post ) else { :error , :create_post_failed } = error -&gt; Logger . error ( &quot;Failed to create post for user \#{ user_id } &quot; ) error { :error , error } = error -&gt; Logger . error ( error ) error end Note When with is used on its own the return of the user step is not available for error handling if create_post fails To match a specific error in else requires workarounds e.g. create_post/2 returning a special error tuple defmodule Blog do def create_post ( %{ user : user , content : content } , opts ) do upcase = Keyword . get ( opts , :upcase , false ) if upcase do insert_post ( user , String . upcase ( content ) ) else insert_post ( user , content ) end end def count_posts ( %{ user : user } ) do { :ok , get_post_count ( user ) } end end ex_que = Q . new ( ) |&gt; Q . put ( :content , &quot;hello world&quot; ) |&gt; Q . run ( :user , fn %{ user_id : id } -&gt; Blog . fetch_user ( id ) end ) |&gt; Q . run ( :post , Blog , :create_post , [ [ upcase : true ] ] ) |&gt; Q . run ( :post_count , &amp; Blog . count_posts / 1 ) with { :ok , %{ post : post } } &lt;- Q . exec ( ex_que ) do broadcast ( post ) else { :error , :post , failed_value , %{ user : user } } -&gt; Logger . error ( &quot;Failed to create post for user \#{ user . id } &quot; ) { :error , failed_value } { :error , failed_operation , failed_value , _changes_so_far } -&gt; Logger . error ( &quot;Operation \#{ failed_operation } failed, \#{ inspect ( failed_value ) } &quot; ) { :error , failed_value } end Note With ExQ the user result is available for error handling if create_post fails We can also handle errors for specific steps while create_post/2 can return a standard error tuple like {:error, &quot;msg&quot;}","ref":"readme.html#comparison-to-with","title":"README - Comparison to with","type":"extras"},{"doc":"Return {:halt, value} to end execution early iex&gt; Q . new ( ) |&gt; Q . put ( :user_id , 1235 ) |&gt; Q . run ( :user , &amp; Blog . fetch_user / 1 end ) |&gt; Q . run ( :earned_daily_award , fn %{ user : user } -&gt; count = Blog . count_posts_today ( user ) if count &lt; 1 do { :halt , false } else { :ok , true } end end ) |&gt; Q . run ( :send_daily_award , Blog , :send_daily_award , [ ] ) |&gt; Q . exec ( )","ref":"readme.html#ending-execution-early","title":"README - Ending execution early","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding ex_q to your list of dependencies in mix.exs : def deps do [ { :ex_q , &quot;~&gt; 1.0&quot; } ] end Documentation can be found at https://hexdocs.pm/ex_q .","ref":"readme.html#installation","title":"README - Installation","type":"extras"}]